package org.apache.bcel.verifier.structurals;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.Vector;
import org.apache.bcel.Repository;
import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.classfile.Method;
import org.apache.bcel.generic.BasicType;
import org.apache.bcel.generic.ConstantPoolGen;
import org.apache.bcel.generic.InstructionHandle;
import org.apache.bcel.generic.JsrInstruction;
import org.apache.bcel.generic.MethodGen;
import org.apache.bcel.generic.ObjectType;
import org.apache.bcel.generic.RET;
import org.apache.bcel.generic.ReferenceType;
import org.apache.bcel.generic.ReturnInstruction;
import org.apache.bcel.generic.ReturnaddressType;
import org.apache.bcel.generic.Type;
import org.apache.bcel.verifier.PassVerifier;
import org.apache.bcel.verifier.VerificationResult;
import org.apache.bcel.verifier.Verifier;
import org.apache.bcel.verifier.exc.AssertionViolatedException;
import org.apache.bcel.verifier.exc.StructuralCodeConstraintException;
import org.apache.bcel.verifier.exc.VerifierConstraintViolatedException;

/* loaded from: org_apache_bcel_verifier_structurals.jar:org/apache/bcel/verifier/structurals/Pass3bVerifier.class */
public final class Pass3bVerifier extends PassVerifier {
    private static final boolean DEBUG = true;
    private final Verifier myOwner;
    private final int methodNo;

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: org_apache_bcel_verifier_structurals.jar:org/apache/bcel/verifier/structurals/Pass3bVerifier$InstructionContextQueue.class */
    public static final class InstructionContextQueue {
        private final List<InstructionContext> ics;
        private final List<ArrayList<InstructionContext>> ecs;

        private InstructionContextQueue() {
            this.ics = new Vector();
            this.ecs = new Vector();
        }

        public void add(InstructionContext ic, ArrayList<InstructionContext> executionChain) {
            this.ics.add(ic);
            this.ecs.add(executionChain);
        }

        public ArrayList<InstructionContext> getEC(int i) {
            return this.ecs.get(i);
        }

        public InstructionContext getIC(int i) {
            return this.ics.get(i);
        }

        public boolean isEmpty() {
            return this.ics.isEmpty();
        }

        public void remove(int i) {
            this.ics.remove(i);
            this.ecs.remove(i);
        }

        public int size() {
            return this.ics.size();
        }
    }

    public Pass3bVerifier(Verifier myOwner, int methodNo) {
        this.myOwner = myOwner;
        this.methodNo = methodNo;
    }

    private void circulationPump(MethodGen m, ControlFlowGraph cfg, InstructionContext start, Frame vanillaFrame, InstConstraintVisitor icv, ExecutionVisitor ev) {
        InstructionHandle next;
        Type returnedType;
        new Random();
        InstructionContextQueue icq = new InstructionContextQueue();
        start.execute(vanillaFrame, new ArrayList<>(), icv, ev);
        icq.add(start, new ArrayList<>());
        while (!icq.isEmpty()) {
            InstructionContext u = icq.getIC(0);
            ArrayList<InstructionContext> ec = icq.getEC(0);
            icq.remove(0);
            ArrayList<InstructionContext> oldchain = (ArrayList) ec.clone();
            ArrayList<InstructionContext> newchain = (ArrayList) ec.clone();
            newchain.add(u);
            if (u.getInstruction().getInstruction() instanceof RET) {
                RET ret = u.getInstruction().getInstruction();
                ReturnaddressType t = u.getOutFrame(oldchain).getLocals().get(ret.getIndex());
                InstructionContext theSuccessor = cfg.contextOf(t.getTarget());
                InstructionContext lastJSR = null;
                int skipJsr = 0;
                int ss = oldchain.size() - DEBUG;
                while (true) {
                    if (ss < 0) {
                        break;
                    }
                    if (skipJsr < 0) {
                        throw new AssertionViolatedException("More RET than JSR in execution chain?!");
                    }
                    if (oldchain.get(ss).getInstruction().getInstruction() instanceof JsrInstruction) {
                        if (skipJsr == 0) {
                            lastJSR = oldchain.get(ss);
                            break;
                        }
                        skipJsr--;
                    }
                    if (oldchain.get(ss).getInstruction().getInstruction() instanceof RET) {
                        skipJsr += DEBUG;
                    }
                    ss--;
                }
                if (lastJSR == null) {
                    throw new AssertionViolatedException("RET without a JSR before in ExecutionChain?! EC: '" + oldchain + "'.");
                }
                JsrInstruction jsr = lastJSR.getInstruction().getInstruction();
                if (theSuccessor != cfg.contextOf(jsr.physicalSuccessor())) {
                    throw new AssertionViolatedException("RET '" + u.getInstruction() + "' info inconsistent: jump back to '" + theSuccessor + "' or '" + cfg.contextOf(jsr.physicalSuccessor()) + "'?");
                }
                if (theSuccessor.execute(u.getOutFrame(oldchain), newchain, icv, ev)) {
                    ArrayList<InstructionContext> newchainClone = (ArrayList) newchain.clone();
                    icq.add(theSuccessor, newchainClone);
                }
            } else {
                InstructionContext[] succs = u.getSuccessors();
                int length = succs.length;
                for (int i = 0; i < length; i += DEBUG) {
                    InstructionContext v = succs[i];
                    if (v.execute(u.getOutFrame(oldchain), newchain, icv, ev)) {
                        ArrayList<InstructionContext> newchainClone2 = (ArrayList) newchain.clone();
                        icq.add(v, newchainClone2);
                    }
                }
            }
            ExceptionHandler[] excHds = u.getExceptionHandlers();
            int length2 = excHds.length;
            for (int i2 = 0; i2 < length2; i2 += DEBUG) {
                ExceptionHandler excHd = excHds[i2];
                InstructionContext v2 = cfg.contextOf(excHd.getHandlerStart());
                if (v2.execute(new Frame(u.getOutFrame(oldchain).getLocals(), new OperandStack(u.getOutFrame(oldchain).getStack().maxStack(), excHd.getExceptionType() == null ? Type.THROWABLE : excHd.getExceptionType())), new ArrayList<>(), icv, ev)) {
                    icq.add(v2, new ArrayList<>());
                }
            }
        }
        InstructionHandle ih = start.getInstruction();
        do {
            if ((ih.getInstruction() instanceof ReturnInstruction) && !cfg.isDead(ih)) {
                InstructionContext ic = cfg.contextOf(ih);
                Frame f = ic.getOutFrame(new ArrayList<>());
                LocalVariables lvs = f.getLocals();
                for (int i3 = 0; i3 < lvs.maxLocals(); i3 += DEBUG) {
                    if (lvs.get(i3) instanceof UninitializedObjectType) {
                        addMessage("Warning: ReturnInstruction '" + ic + "' may leave method with an uninitialized object in the local variables array '" + lvs + "'.");
                    }
                }
                OperandStack os = f.getStack();
                for (int i4 = 0; i4 < os.size(); i4 += DEBUG) {
                    if (os.peek(i4) instanceof UninitializedObjectType) {
                        addMessage("Warning: ReturnInstruction '" + ic + "' may leave method with an uninitialized object on the operand stack '" + os + "'.");
                    }
                }
                OperandStack inStack = ic.getInFrame().getStack();
                if (inStack.size() >= DEBUG) {
                    returnedType = inStack.peek();
                } else {
                    returnedType = Type.VOID;
                }
                if (returnedType != null) {
                    if (returnedType instanceof ReferenceType) {
                        try {
                            if (!((ReferenceType) returnedType).isCastableTo(m.getReturnType())) {
                                invalidReturnTypeError(returnedType, m);
                            }
                        } catch (ClassNotFoundException e) {
                            throw new IllegalArgumentException(e);
                        }
                    } else if (!returnedType.equals(m.getReturnType().normalizeForStackOrLocal())) {
                        invalidReturnTypeError(returnedType, m);
                    }
                }
            }
            next = ih.getNext();
            ih = next;
        } while (next != null);
    }

    public VerificationResult do_verify() {
        if (!this.myOwner.doPass3a(this.methodNo).equals(VerificationResult.VR_OK)) {
            return VerificationResult.VR_NOTYET;
        }
        try {
            JavaClass jc = Repository.lookupClass(this.myOwner.getClassName());
            ConstantPoolGen constantPoolGen = new ConstantPoolGen(jc.getConstantPool());
            InstConstraintVisitor icv = new InstConstraintVisitor();
            icv.setConstantPoolGen(constantPoolGen);
            ExecutionVisitor ev = new ExecutionVisitor();
            ev.setConstantPoolGen(constantPoolGen);
            Method[] methods = jc.getMethods();
            try {
                MethodGen mg = new MethodGen(methods[this.methodNo], this.myOwner.getClassName(), constantPoolGen);
                icv.setMethodGen(mg);
                if (!mg.isAbstract() && !mg.isNative()) {
                    ControlFlowGraph cfg = new ControlFlowGraph(mg);
                    Frame f = new Frame(mg.getMaxLocals(), mg.getMaxStack());
                    if (!mg.isStatic()) {
                        if (mg.getName().equals("<init>")) {
                            Frame.setThis(new UninitializedObjectType(ObjectType.getInstance(jc.getClassName())));
                            f.getLocals().set(0, Frame.getThis());
                        } else {
                            Frame.setThis(null);
                            f.getLocals().set(0, ObjectType.getInstance(jc.getClassName()));
                        }
                    }
                    BasicType[] argumentTypes = mg.getArgumentTypes();
                    int twoslotoffset = 0;
                    for (int j = 0; j < argumentTypes.length; j += DEBUG) {
                        if (argumentTypes[j] == Type.SHORT || argumentTypes[j] == Type.BYTE || argumentTypes[j] == Type.CHAR || argumentTypes[j] == Type.BOOLEAN) {
                            argumentTypes[j] = Type.INT;
                        }
                        f.getLocals().set(twoslotoffset + j + (mg.isStatic() ? 0 : DEBUG), argumentTypes[j]);
                        if (argumentTypes[j].getSize() == 2) {
                            twoslotoffset += DEBUG;
                            f.getLocals().set(twoslotoffset + j + (mg.isStatic() ? 0 : DEBUG), Type.UNKNOWN);
                        }
                    }
                    circulationPump(mg, cfg, cfg.contextOf(mg.getInstructionList().getStart()), f, icv, ev);
                }
                return VerificationResult.VR_OK;
            } catch (RuntimeException re) {
                StringWriter sw = new StringWriter();
                PrintWriter pw = new PrintWriter(sw);
                re.printStackTrace(pw);
                throw new AssertionViolatedException("Some RuntimeException occurred while verify()ing class '" + jc.getClassName() + "', method '" + methods[this.methodNo] + "'. Original RuntimeException's stack trace:\n---\n" + sw + "---\n", re);
            } catch (VerifierConstraintViolatedException ce) {
                ce.extendMessage("Constraint violated in method '" + methods[this.methodNo] + "':\n", "");
                return new VerificationResult(2, ce.getMessage());
            }
        } catch (ClassNotFoundException e) {
            throw new AssertionViolatedException("Missing class: " + e, e);
        }
    }

    public int getMethodNo() {
        return this.methodNo;
    }

    public void invalidReturnTypeError(Type returnedType, MethodGen m) {
        throw new StructuralCodeConstraintException("Returned type " + returnedType + " does not match Method's return type " + m.getReturnType());
    }
}
